import 'package:flutter/material.dart';

class DifficultyToggle extends StatefulWidget {
  final String startingDifficulty;
  final ValueChanged<String> onValueClick;
  final TextStyle valueFontStyle;
  final double spacing;
  final Duration animationDuration;
  final double size;
  final double borderRadiusOutside;
  final double borderRadiusInside;
  final Color colorBackground;
  final Color colorSlider;
  final List<BoxShadow> boxShadow;

  DifficultyToggle({
    @required this.startingDifficulty,
    @required this.onValueClick,
    this.valueFontStyle,
    this.spacing = 4.0,
    this.size = 40,
    this.borderRadiusInside,
    this.borderRadiusOutside,
    this.colorBackground = const Color(0xFFE5E5E5),
    this.colorSlider = Colors.white,
    this.animationDuration = const Duration(milliseconds: 200),
    this.boxShadow,
  })  : assert(startingDifficulty != null),
        assert(onValueClick != null);

  @override
  State<StatefulWidget> createState() {
    return _DifficultyToggleState();
  }
}

class _DifficultyToggleState extends State<DifficultyToggle> {
  final List<Map<String, String>> levels = [
    {"key": "easy", "name": "Easy"},
    {"key": "medium", "name": "Medium"},
    {"key": "hard", "name": "Hard"},
  ];

  String currentDifficulty;

  Widget values() {
    return Container(
      height: double.infinity,
      padding: EdgeInsets.all(widget.spacing),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisSize: MainAxisSize.max,
        children: levels.map<Widget>(
          (element) {
            return Expanded(
              child: GestureDetector(
                behavior: HitTestBehavior.opaque,
                child: Container(
                  width: double.infinity,
                  alignment: Alignment.center,
                  child: Text(
                    element["name"],
                    style: widget.valueFontStyle,
                  ),
                ),
                onTap: () {
                  setState(() {
                    currentDifficulty = element["key"];
                  });
                  widget.onValueClick(element["key"]);
                },
              ),
            );
          },
        ).toList(),
      ),
    );
  }

  AlignmentGeometry getCurrentAlignment() {
    var chosen = currentDifficulty ?? widget.startingDifficulty;

    switch (chosen) {
      case "easy":
        return Alignment.centerLeft;
        break;
      case "medium":
        return Alignment.center;
        break;
      case "hard":
        return Alignment.centerRight;
        break;
      default:
        return Alignment.centerLeft;
        break;
    }
  }

  Widget box() {
    return Container(
      width: double.infinity,
      child: LayoutBuilder(
        builder: (cont, cons) => AnimatedAlign(
          alignment: getCurrentAlignment(),
          curve: Curves.easeInOut,
          heightFactor: 1,
          widthFactor: 1 / 3,
          duration: widget.animationDuration,
          child: Container(
            height: double.infinity,
            width: cons.biggest.width / 3,
            margin: EdgeInsets.all(widget.spacing),
            decoration: BoxDecoration(
              color: widget.colorSlider,
              boxShadow: widget.boxShadow ??
                  [
                    BoxShadow(
                      color: Color(0xFF000000).withOpacity(0.12),
                      offset: Offset(0, 2),
                      blurRadius: 12.0,
                      spreadRadius: 0,
                    ),
                  ],
              borderRadius: BorderRadius.circular(
                widget.borderRadiusInside ?? widget.size,
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: widget.size,
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(
          widget.borderRadiusOutside ?? widget.size,
        ),
        color: widget.colorBackground,
      ),
      child: Stack(
        clipBehavior: Clip.none,
        children: [
          box(),
          values(),
        ],
      ),
    );
  }
}
